
#include-once
#include 'GeometryShapes.au3'
;~ #include 'C:\Users\pluto\OneDrive\AutoIt\UDF\Map.au3'
#include <Array.au3>
#include 'FastHSL.au3'

;~ $a = A3D_GeometryIcosahedron()
;~ $a = A3D_GeometryTetrahedron()
;~ _GeometrySubdiv($a)

Func _GeometryColorSet(ByRef $aGeo, $iCol)
	Local $aHSL = _ColorIA_RGB2HSL($iCol), $_[UBound($aGeo[1])]
	For $i = 0 To UBound($_) - 1 Step 1
		$_[$i] = $aHSL
	Next
	$aGeo[2] = $_
EndFunc

Func _GeometryTranslate(ByRef $aGeo, $dx, $dy, $dz)
	__GeometryTranslate($aGeo[0], $dx, $dy, $dz)
EndFunc   ;==>_GeometryTranslate

Func _GeometryScale(ByRef $aGeo, $mx, $my, $mz)
	__GeometryScale($aGeo[0], $mx, $my, $mz)
EndFunc   ;==>_GeometryScale

Func _GeometryProjectOnSphere(ByRef $aGeo, $nSize = 1.0, $bDudeRandomMan = 0, $nAmount = 1.0)
	__GeometryProjectOnSphere($aGeo[0], $nSize, $bDudeRandomMan, $nAmount)
EndFunc   ;==>_GeometryProjectOnSphere

Func _GeometryGetBoundingBox(ByRef $aGeo)
	Return __GeometryGetBoundingBox($aGeo[0])
EndFunc   ;==>__GeometryGetBoundingBox

Func _GeometrySubdiv(ByRef $aGeo)
	__GeometrySubdiv($aGeo[0], $aGeo[1])
EndFunc

Func _GeometryToOFF(ByRef $aGeo)
	; Aktuell nur für Trianglemeshes implementiert.
	; Aktuell keine Farben
	; OFF unterstützt nicht immer Edges
	; OFF unterstützt nicht immer VertexColors
	; OFF unterstützt nicht immer TriangleColors
	Local $aVertices = $aGeo[0], $aTriangles = $aGeo[1], $sOFF = 'OFF' & @CRLF & UBound($aVertices) & ' ' & UBound($aTriangles) & ' 0' & @CRLF
	For $i = 0 To UBound($aVertices) - 1 Step 1
		$sOFF &= StringFormat(' %9.6f', $aVertices[$i][0]) & ' ' & StringFormat(' %9.6f', $aVertices[$i][1]) & ' ' & StringFormat(' %9.6f', $aVertices[$i][2]) & @CRLF
	Next
	For $i = 0 To UBound($aTriangles) - 1 Step 1
		$sOFF &= '3 ' & $aTriangles[$i][0] & ' ' & $aTriangles[$i][1] & ' ' & $aTriangles[$i][2] & @CRLF
	Next
	Return $sOFF
EndFunc   ;==>_GeometryToOFF

Func __GeometrySubdiv(ByRef $aVertices, ByRef $aIndices)
	Switch UBound($aIndices, 2)
		Case $GEO_TYPE_POINT
			Return ; Punkte kann man nicht unterteilen
		Case $GEO_TYPE_LINE
			__GeometrySubdivLine($aVertices, $aIndices)
		Case $GEO_TYPE_TRIANGLE
			__GeometrySubdivTriangle($aVertices, $aIndices)
		Case $GEO_TYPE_QUAD
			__GeometrySubdivQuad($aVertices, $aIndices)
	EndSwitch
EndFunc

Func __GeometrySubdivLine(ByRef $aV, ByRef $aI)

	Local $iLutSize = 8, $aBB = 0, $aLut = __GeometryVertexLut($iLutSize)
	__GeometryRescale($aV, $aBB, $iLutSize)

	Local $aVS[UBound($aI, 1) * 3][3] ; 1 Line -> 3 Vertices
	Local $aIS[UBound($aI, 1) * 2][UBound($aI, 2)]              ; 1 Line -> 2 Lines
	Local $ii = 0, $iv = 0

	For $i = 0 To UBound($aI, 1) - 1 Step 1
		Local $av0 = [$aV[$aI[$i][0]][0], $aV[$aI[$i][0]][1], $aV[$aI[$i][0]][2], 0]
		Local $av1 = [$aV[$aI[$i][1]][0], $aV[$aI[$i][1]][1], $aV[$aI[$i][1]][2], 0]
		Local $av2 = [($av0[0] + $av1[0]) / 2, ($av0[1] + $av1[1]) / 2, ($av0[2] + $av1[2]) / 2, 0]

		$av0[3] = __GeometryVertexLutFind($aLut, $av0)
		If $av0[3] = -1 Then
			$av0[3] = $iv
			__GeometryVertexLutInsert($aLut, $av0)
			$iv += 1
			$aVS[$av0[3]][0] = $av0[0]
			$aVS[$av0[3]][1] = $av0[1]
			$aVS[$av0[3]][2] = $av0[2]
		EndIf

		$av1[3] = __GeometryVertexLutFind($aLut, $av1)
		If $av1[3] = -1 Then
			$av1[3] = $iv
			__GeometryVertexLutInsert($aLut, $av1)
			$iv += 1
			$aVS[$av1[3]][0] = $av1[0]
			$aVS[$av1[3]][1] = $av1[1]
			$aVS[$av1[3]][2] = $av1[2]
		EndIf

		$av2[3] = __GeometryVertexLutFind($aLut, $av2)
		If $av2[3] = -1 Then
			$av2[3] = $iv
			__GeometryVertexLutInsert($aLut, $av2)
			$iv += 1
			$aVS[$av2[3]][0] = $av2[0]
			$aVS[$av2[3]][1] = $av2[1]
			$aVS[$av2[3]][2] = $av2[2]
		EndIf

		$aIS[$ii + 0][0] = $av0[3]
		$aIS[$ii + 0][1] = $av2[3]
		$aIS[$ii + 1][0] = $av2[3]
		$aIS[$ii + 1][1] = $av1[3]

		$ii += 2
	Next

	ReDim $aVS[$iv][3]

	$aV = $aVS
	$aI = $aIS

	__GeometryRescale($aV, $aBB)

;~ 	_ArrayDisplay($aI)

EndFunc

Func __GeometrySubdivTriangle(ByRef $aV, ByRef $aI)

	Local $iLutSize = 8, $aBB = 0, $aLut = __GeometryVertexLut($iLutSize)
	__GeometryRescale($aV, $aBB, $iLutSize)

	Local $aVS[UBound($aI, 1) * 6][3] ; 1 Dreieck -> 6 Vertices
	Local $aIS[UBound($aI, 1) * 4][UBound($aI, 2)]              ; 1 Dreieck -> 4 Dreiecke
	Local $ii = 0, $iv = 0

	For $i = 0 To UBound($aI, 1) - 1 Step 1

		Local $av0 = [$aV[$aI[$i][0]][0], $aV[$aI[$i][0]][1], $aV[$aI[$i][0]][2], 0]
		Local $av1 = [$aV[$aI[$i][1]][0], $aV[$aI[$i][1]][1], $aV[$aI[$i][1]][2], 0]
		Local $av2 = [$aV[$aI[$i][2]][0], $aV[$aI[$i][2]][1], $aV[$aI[$i][2]][2], 0]
		Local $av3 = [($av0[0] + $av1[0]) / 2, ($av0[1] + $av1[1]) / 2, ($av0[2] + $av1[2]) / 2, 0]
		Local $av4 = [($av1[0] + $av2[0]) / 2, ($av1[1] + $av2[1]) / 2, ($av1[2] + $av2[2]) / 2, 0]
		Local $av5 = [($av0[0] + $av2[0]) / 2, ($av0[1] + $av2[1]) / 2, ($av0[2] + $av2[2]) / 2, 0]

		$av0[3] = __GeometryVertexLutFind($aLut, $av0)
		If $av0[3] = -1 Then
			$av0[3] = $iv
			__GeometryVertexLutInsert($aLut, $av0)
			$iv += 1
			$aVS[$av0[3]][0] = $av0[0]
			$aVS[$av0[3]][1] = $av0[1]
			$aVS[$av0[3]][2] = $av0[2]
		EndIf

		$av1[3] = __GeometryVertexLutFind($aLut, $av1)
		If $av1[3] = -1 Then
			$av1[3] = $iv
			__GeometryVertexLutInsert($aLut, $av1)
			$iv += 1
			$aVS[$av1[3]][0] = $av1[0]
			$aVS[$av1[3]][1] = $av1[1]
			$aVS[$av1[3]][2] = $av1[2]
		EndIf

		$av2[3] = __GeometryVertexLutFind($aLut, $av2)
		If $av2[3] = -1 Then
			$av2[3] = $iv
			__GeometryVertexLutInsert($aLut, $av2)
			$iv += 1
			$aVS[$av2[3]][0] = $av2[0]
			$aVS[$av2[3]][1] = $av2[1]
			$aVS[$av2[3]][2] = $av2[2]
		EndIf

		$av3[3] = __GeometryVertexLutFind($aLut, $av3)
		If $av3[3] = -1 Then
			$av3[3] = $iv
			__GeometryVertexLutInsert($aLut, $av3)
			$iv += 1
			$aVS[$av3[3]][0] = $av3[0]
			$aVS[$av3[3]][1] = $av3[1]
			$aVS[$av3[3]][2] = $av3[2]
		EndIf

		$av4[3] = __GeometryVertexLutFind($aLut, $av4)
		If $av4[3] = -1 Then
			$av4[3] = $iv
			__GeometryVertexLutInsert($aLut, $av4)
			$iv += 1
			$aVS[$av4[3]][0] = $av4[0]
			$aVS[$av4[3]][1] = $av4[1]
			$aVS[$av4[3]][2] = $av4[2]
		EndIf

		$av5[3] = __GeometryVertexLutFind($aLut, $av5)
		If $av5[3] = -1 Then
			$av5[3] = $iv
			__GeometryVertexLutInsert($aLut, $av5)
			$iv += 1
			$aVS[$av5[3]][0] = $av5[0]
			$aVS[$av5[3]][1] = $av5[1]
			$aVS[$av5[3]][2] = $av5[2]
		EndIf

		$aIS[$ii + 0][0] = $av1[3]
		$aIS[$ii + 0][1] = $av4[3]
		$aIS[$ii + 0][2] = $av3[3]
		$aIS[$ii + 1][0] = $av0[3]
		$aIS[$ii + 1][1] = $av3[3]
		$aIS[$ii + 1][2] = $av5[3]
		$aIS[$ii + 2][0] = $av2[3]
		$aIS[$ii + 2][1] = $av5[3]
		$aIS[$ii + 2][2] = $av4[3]
		$aIS[$ii + 3][0] = $av3[3]
		$aIS[$ii + 3][1] = $av4[3]
		$aIS[$ii + 3][2] = $av5[3]

		$ii += 4
	Next

	ReDim $aVS[$iv][UBound($aI, 2)]

	$aV = $aVS
	$aI = $aIS

	__GeometryRescale($aV, $aBB)

EndFunc

Func __GeometrySubdivQuad(ByRef $aVertices, ByRef $aIndices)
EndFunc




Func _GeometryConvert(ByRef $aGeo, $iType = $GEO_TYPE_POINT)
	Switch $iType
		Case $GEO_TYPE_POINT
			Return __GeometryConvertP($aGeo[0], $aGeo[1])
		Case $GEO_TYPE_LINE
		Case $GEO_TYPE_TRIANGLE
		Case $GEO_TYPE_QUAD
	EndSwitch
EndFunc

Func __GeometryConvertP(ByRef $aV, $aI)
	Local $aIndices[UBound($aV, 1)][1]
	For $i = 0 To UBound($aIndices) - 1 Step 1
		$aIndices[$i][0] = $i
	Next
	Local $aGeo = [$aV, $aIndices]
	Return $aGeo
EndFunc

Func __GeometryScale(ByRef $aV, $mx, $my, $mz)
	For $i = 0 To UBound($aV) - 1 Step 1
		$aV[$i][0] *= $mx
		$aV[$i][1] *= $my
		$aV[$i][2] *= $mz
	Next
EndFunc   ;==>__GeometryScale

Func __GeometryProjectOnSphere(ByRef $aV, $nSize, $bDudeRandomMan, $nAmount)
	Local $l = 1.0
	For $i = 0 To UBound($aV, 1) - 1 Step 1
		$l = Sqrt($aV[$i][0] ^ 2 + $aV[$i][1] ^ 2 + $aV[$i][2] ^ 2) / $nSize + ($bDudeRandomMan ? Random(-$bDudeRandomMan, $bDudeRandomMan) : 0)
		$aV[$i][0] = $nAmount * $aV[$i][0] / $l + (1 - $nAmount) * $aV[$i][0]
		$aV[$i][1] = $nAmount * $aV[$i][1] / $l + (1 - $nAmount) * $aV[$i][1]
		$aV[$i][2] = $nAmount * $aV[$i][2] / $l + (1 - $nAmount) * $aV[$i][2]
;~ 		$aV[$i][0] /= $l
;~ 		$aV[$i][1] /= $l
;~ 		$aV[$i][2] /= $l
	Next
EndFunc   ;==>__GeometryProjectOnSphere

Func __GeometryTranslate(ByRef $aV, $dx, $dy, $dz)
	For $i = 0 To UBound($aV) - 1 Step 1
		$aV[$i][0] += $dx
		$aV[$i][1] += $dy
		$aV[$i][2] += $dz
	Next
EndFunc   ;==>__GeometryTranslate

Func __GeometryGetBoundingBox(ByRef $aV)
	Local $aMin = [$aV[0][0], $aV[0][1], $aV[0][2]], $aMax = $aMin ; x,y,z,dx,dy,dz
	For $i = 1 To UBound($aV) - 1 Step 1
		If $aV[$i][0] < $aMin[0] Then $aMin[0] = $aV[$i][0]
		If $aV[$i][1] < $aMin[1] Then $aMin[1] = $aV[$i][1]
		If $aV[$i][2] < $aMin[2] Then $aMin[2] = $aV[$i][2]
		If $aV[$i][0] > $aMax[0] Then $aMax[0] = $aV[$i][0]
		If $aV[$i][1] > $aMax[1] Then $aMax[1] = $aV[$i][1]
		If $aV[$i][2] > $aMax[2] Then $aMax[2] = $aV[$i][2]
	Next
	Local $aRet = [$aMin[0], $aMin[1], $aMin[2], $aMax[0] - $aMin[0], $aMax[1] - $aMin[1], $aMax[2] - $aMin[2]]
	Return $aRet
EndFunc   ;==>__GeometryGetBoundingBox

Func __GeometryRescale(ByRef $aV, ByRef $aBB, $iSize = 16)
	If IsArray($aBB) Then ; Skaliert das Objekt so dass die Werte bei 0 beginnen und bei $iSize - 1 enden
		For $i = 0 To UBound($aV) - 1 Step 1 ; Zurück abbilden
			$aV[$i][0] = $aV[$i][0] * $aBB[3] + $aBB[0]
			$aV[$i][1] = $aV[$i][1] * $aBB[4] + $aBB[1]
			$aV[$i][2] = $aV[$i][2] * $aBB[5] + $aBB[2]
		Next
	Else
		$aBB = __GeometryGetBoundingBox($aV); x,y,z,dx,dy,dz
		$aBB[3] /= $iSize - 1
		$aBB[4] /= $iSize - 1
		$aBB[5] /= $iSize - 1
		For $i = 0 To UBound($aV) - 1 Step 1 ; Abbildung auf Einheitswürfel
			$aV[$i][0] = ($aV[$i][0] - $aBB[0]) / $aBB[3]
			$aV[$i][1] = ($aV[$i][1] - $aBB[1]) / $aBB[4]
			$aV[$i][2] = ($aV[$i][2] - $aBB[2]) / $aBB[5]
		Next
	EndIf
EndFunc

Func __GeometryVertexLut($iSize)
	Local $m[], $aXYZ[$iSize][$iSize][$iSize]
	$m.Size = 0
	For $z = 0 To $iSize - 1 Step 1
		For $y = 0 To $iSize - 1 Step 1
			For $x = 0 To $iSize - 1 Step 1
				$aXYZ[$x][$y][$z] = $m
			Next
		Next
	Next
	Return $aXYZ
EndFunc

Func __GeometryVertexLutFind(ByRef $aLut, ByRef $v0)
	Local $mV = $aLut[$v0[0]][$v0[1]][$v0[2]]
	If $mV.Size = 0 Then Return -1
	Local $v1
	For $i = 0 To $mV.Size - 1 Step 1
		$v1 = $mV[$i]
		If ($v0[0] - $v1[0])^2 + ($v0[1] - $v1[1])^2 + ($v0[2] - $v1[2])^2 < 1e-9 Then Return $v1[3]
	Next
	Return -1
EndFunc

Func __GeometryVertexLutInsert(ByRef $aLut, ByRef $v0)
	MapAppend($aLut[$v0[0]][$v0[1]][$v0[2]], $v0)
	$aLut[$v0[0]][$v0[1]][$v0[2]].Size += 1
EndFunc



