#include-once
#include 'GeometryConstants.au3'

; Alle punktsymmetrischen geometrischen Körper sind um den Nullpunkt zentriert und haben den Umkreisdurchmesser 1.0

Func _GeometryIcosahedron($iType = $GEO_TYPE_TRIANGLE, $iColorRGB = 0xFF8CBAE1)
	If $iType = $GEO_TYPE_QUAD Then Return Null ; Geht nicht.
	Local $_ = (1.0 + Sqrt(5.0)) / 2.0, $l = Sqrt($_ * $_ + 1), $a = 1 / $l, $b = $_ / $l
	Local $aVertices = [[0, -$a, -$b], [0, -$a, $b], [0, $a, -$b], [0, $a, $b], [-$a, -$b, 0], [-$a, $b, 0], [$a, -$b, 0], [$a, $b, 0], [-$b, 0, -$a], [-$b, 0, $a], [$b, 0, -$a], [$b, 0, $a]]
	Switch $iType
		Case $GEO_TYPE_POINT
			Local $aIndices = [[0], [1], [2], [3], [4], [5], [6], [7], [8], [9], [10], [11]]
		Case $GEO_TYPE_LINE
			Local $aIndices = [[0, 2], [0, 4], [0, 6], [0, 8], [0, 10], [1, 3], [1, 4], [1, 6], [1, 9], [1, 11], [2, 5], [2, 7], [2, 8], [2, 10], [3, 5], [3, 7], [3, 9], [3, 11], [4, 6], [4, 8], [4, 9], [5, 7], [5, 8], [5, 9], [6, 10], [6, 11], [7, 10], [7, 11], [8, 9], [10, 11]]
		Case $GEO_TYPE_TRIANGLE
			Local $aIndices = [[0, 2, 10], [0, 4, 8], [0, 6, 4], [0, 8, 2], [0, 10, 6], [1, 3, 9], [1, 11, 3], [1, 9, 4], [1, 4, 6], [1, 6, 11], [2, 5, 7], [2, 8, 5], [2, 7, 10], [3, 7, 5], [3, 5, 9], [3, 11, 7], [4, 9, 8], [5, 8, 9], [6, 10, 11], [7, 11, 10]]
	EndSwitch
	Local $aColors[0], $aGeo = [$aVertices, $aIndices, $aColors]
	Return $aGeo
EndFunc   ;==>A3D_GeometryIcosahedron

Func _GeometryCube($iType = $GEO_TYPE_TRIANGLE, $iColorRGB = 0xFF8CBAE1)
	Local $_ = 0.5, $aVertices = [[-$_, -$_, -$_], [-$_, -$_, $_], [$_, -$_, $_], [$_, -$_, -$_], [-$_, $_, $_], [$_, $_, $_], [$_, $_, -$_], [-$_, $_, -$_]]
	Switch $iType
		Case $GEO_TYPE_POINT
			Local $aIndices = [[0], [1], [2], [3], [4], [5], [6], [7]]
		Case $GEO_TYPE_LINE
			Local $aIndices = [[0, 1], [0, 3], [0, 7], [1, 2], [1, 4], [2, 3], [2, 5], [3, 6], [4, 5], [4, 7], [5, 6], [6, 7]]
		Case $GEO_TYPE_TRIANGLE
			Local $aIndices = [[0, 1, 4], [0, 4, 7], [1, 5, 4], [1, 2, 5], [2, 3, 6], [2, 6, 5], [0, 3, 2], [0, 2, 1], [0, 7, 3], [3, 7, 6], [4, 5, 7], [5, 6, 7]]
		Case $GEO_TYPE_QUAD
			Local $aIndices = [[1, 0, 3, 2], [2, 3, 6, 5], [0, 1, 4, 7], [2, 5, 4, 1], [6, 3, 0, 7], [4, 5, 6, 7]]
	EndSwitch
	Local $aColors[0], $aGeo = [$aVertices, $aIndices, $aColors]
	Return $aGeo
EndFunc   ;==>A3D_GeometryCube

Func _GeometryTetrahedron($iType = $GEO_TYPE_TRIANGLE, $iColorRGB = 0xFF8CBAE1)
	If $iType = $GEO_TYPE_QUAD Then Return Null ; Geht nicht.
	Local $_ = 1.0 / Sqrt(3.0), $aVertices = [[-$_, -$_, -$_], [$_, -$_, $_], [-$_, $_, $_], [$_, $_, -$_]]
	Switch $iType
		Case $GEO_TYPE_POINT
			Local $aIndices = [[0], [1], [2], [3]]
		Case $GEO_TYPE_LINE
			Local $aIndices = [[0, 1], [1, 3], [2, 3], [3, 0], [1, 2], [0, 2]]
		Case $GEO_TYPE_TRIANGLE
			Local $aIndices = [[0, 2, 3], [2, 1, 3], [1, 2, 0], [0, 3, 1]]
	EndSwitch
	Local $aColors[0], $aGeo = [$aVertices, $aIndices, $aColors]
	Return $aGeo
EndFunc   ;==>A3D_GeometryTetrahedron
